#ifndef BUTTONS_H
#define BUTTONS_H

#define BUTTON1		(PINB & (1<<4))	// forward button
#define BUTTON2		(PINB & (1<<5))	// backward button




/*****************************************************************************
 * Function: BtnInit
 *
 * Preconditon: None.
 *
 * Overview: Setup debounce.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void BtnInit(void);

/*****************************************************************************
 * Function: BtnProcessEvents
 *
 * Preconditon: None.
 *
 * Overview: Must be called periodically to proccess buttons.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void BtnProcessEvents(void);


/*****************************************************************************
 * Structure: BUTTONS _button_press
 *
 * Overview: the structure provides a n access to button bit indicators.
 *
 *****************************************************************************/
typedef struct tagButton {
	volatile unsigned b1:1;
	volatile unsigned b2:1;
}BUTTON;
extern BUTTON _button_press;

/*****************************************************************************
 * Debounce button counters
 *
 *****************************************************************************/
#define	BUTTON_MAX_DEBOUCE 	20
extern volatile unsigned char _b1_cnt;
extern volatile unsigned char _b2_cnt;


/*****************************************************************************
 * Function: BtnIsPressed
 *
 * Preconditon: None.
 *
 * Overview: Macro detects if button is pressed
 *
 * Input: None.
 *
 * Output: Macro returns zero if button is not pressed.
 *
 *****************************************************************************/
#define BtnIsPressed(__btn)	_button_press.b##__btn

/*****************************************************************************
 * Function: BtnIsNotPressed
 *
 * Preconditon: None.
 *
 * Overview: Macro detects if button is not pressed
 *
 * Input: None.
 *
 * Output: Macro returns zero if button is pressed.
 *
 *****************************************************************************/
#define BtnIsNotPressed(__btn)	!_button_press.b##__btn

#endif
