#include <avr/io.h>             
#include <timer.h>
#include <ad.h>
#include <powerstage.h>
#include <es_seek.h>


volatile char unsigned State;													
volatile char unsigned Adstore;					
volatile char unsigned Adbuffer0;				
volatile char unsigned Adbuffer1;					
volatile char unsigned Diffvalue;
					
ES_SEEK _es_seek;					



/*****************************************************************************
 * Function: ElectronicSeek_Init
 *
 * Preconditon: None.
 *
 * Overview:
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/	
void ElectronicSeek_Init()
{
	
	DDRA &=~ (1<<0);
	DDRA &=~ (1<<1);
	ADMUX = 0xab;
	State = 0;
}
/*****************************************************************************
 * Function: ElectronicSeek_Statemachine
 *
 * Preconditon: None.
 *
 * Overview:
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/	
void ElectronicSeek_Statemachine()
{

	const char unsigned Delta_1		= 	8; 
	const char unsigned Delta_2		= 	24;   
	const char unsigned Delta_3		= 	30;									



	if (State == 1)				
	{
		if (Adstore < Voltage)
		{
			Adstore = Voltage;	
			Diffvalue = 0;
		}
		if ((Adstore - Voltage) >= Delta_2)
		{
			State = 2;
			Adstore = 0xff;			
		}	
		ElectronicSeek_changeFreq();
		return;
	}

	if (State == 2)				
	{
		if (Voltage < Adstore)
		{
			Adstore = Voltage;
		}
		if (Adstore < (Voltage - Delta_1))
		{
			State = 0;
		}
		ElectronicSeek_changeFreq();
		return;
	}
	
	if (State == 3)				
	{
		if (Adstore < Voltage)
		{
			Adstore = Voltage;
		}
		if ((Adstore - Voltage) >= Delta_3)
		{
			State = 4;  
			Adbuffer0 = 0xff;
			Adbuffer1 = 0xff;
			Adstore = 0xff;
			Diffvalue = 0;
		}
		ElectronicSeek_changeFreq();
		return;
	}
	
	if (State == 4)				
	{
		if (Voltage < Adstore)
		{
			Adstore = Voltage;
			Diffvalue = 0;
		}
		if ((Voltage - Adstore) >= Delta_2)
		{
			State = 5;
			Adstore = 0;
		}
		ElectronicSeek_changeFreq();
		return;
	}

	if (State == 5)				
	{
		if (Adstore < Voltage)
		{
			Adstore = Voltage;
		}
		if (Voltage < (Adstore - Delta_1))
		{
			State = 0;	
		}
	}
	ElectronicSeek_changeFreq();
	return;
}


/*****************************************************************************
 * Function: ElectronicSeek
 *
 * Preconditon: ElectronicSeek_Init has to be called befor using this function
 *				AD_Init has to be called before using this function
 *
 * Overview:
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/	
void ElectronicSeek()
{
	_powerstage.Period = MaxPeriod;
	_powerstage.Power = 0x7f;
	Powerstage_SetPWM();
	delay_time_10ms = 5;
	while (delay_time_10ms != 0);
	Diffvalue = 0;
	Adbuffer0 = 0;
	Adbuffer1 = 0;
	Adstore = 0;
	State = 1;
	while (State != 0);
	_powerstage.Power = 0x00;
	Powerstage_SetPWM();


	Adbuffer0 = 0xff;
	Adbuffer1 = 0xff;
	Diffvalue = 0;
	_powerstage.Period = MinPeriod;
	_powerstage.Power = 0x7f;
	Powerstage_SetPWM();
	delay_time_10ms = 5;
	while (delay_time_10ms != 0);
	Adstore = 0;
	State = 3;
	while (State != 0);
	_powerstage.Power = 0x00;
	Powerstage_SetPWM();
	
}


/*****************************************************************************
 * Function: ElectronicSeek_changeFreq
 *
 * Preconditon: None.
 *
 * Overview:
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/	
void ElectronicSeek_changeFreq()
{
	char unsigned DiffValue2;
	if (State == 0)
	{
		return;
	}
	if (State == 1 || State == 2)
	{
		_powerstage.Period -=1;
		if (Voltage < Adbuffer0)
		{
			if ((Adbuffer0 - Voltage) >= Diffvalue)
			{
			Diffvalue = (Adbuffer0 - Voltage);
			_es_seek.FreqFW = _powerstage.Period + 2;
			}
		}
	}
	else
	{
		_powerstage.Period +=1;
		if (Adbuffer0 < Voltage)
		{
			DiffValue2 = Voltage - Adbuffer0;
			if ((Voltage / 4) >= DiffValue2)
			{
				DiffValue2 = 0;
			}
			else
			{
			DiffValue2 = DiffValue2 - (Voltage / 4);
			}
		
			if (DiffValue2 >= Diffvalue)
			{
				Diffvalue = DiffValue2;
				_es_seek.FreqBW = _powerstage.Period - 2;	
			}
		}
	}
	Powerstage_SetPWM();
	Adbuffer0 = Adbuffer1;
	Adbuffer1 = Voltage;
}











